var statisticsData = {};

AJS.$(document).ready(function() {

    var TableColumnsEnum = {
        userKey: 0,
        fullname: 1,
        username: 2,
        startDate: 3,
        endDate: 4,
        availableDays: 5,
        carryOverDays: 6,
        usedDays: 7,
        availableDaysOrderData: 8,
        carryOverDaysOrderData: 9,
        usedDaysOrderData: 10,
        documents: 11,
        documentsOrderData: 12
    };
	
   	var filter=new TableFilter("dropdownButton", "vacman-filters", table);
   	filter.addFilterDefinition(AJS.I18n.getText("vacationmanager.action.statistics.filter.name"), TableColumnsEnum.fullname, "text", true);
   	filter.addFilterDefinition(AJS.I18n.getText("vacationmanager.action.statistics.filter.available.days"), TableColumnsEnum.availableDaysOrderData, "num", false);
   	filter.addFilterDefinition(AJS.I18n.getText("vacationmanager.action.statistics.filter.carryover.days"), TableColumnsEnum.carryOverDaysOrderData, "num", false);
   	filter.addFilterDefinition(AJS.I18n.getText("vacationmanager.action.statistics.filter.used.days"), TableColumnsEnum.usedDaysOrderData, "num", false);
   	filter.setUpdateCallback(function() {
   		table.rows().invalidate();
   		table.draw();
   	});

	var checkDateStart=new Date();
	var checkDateEnd=new Date(new Date().getFullYear(), 11, 31);
	var vacationKindId=0;
	
	AJS.$('.aui-date-picker').each(function() {
		AJS.$(this).datePicker({'overrideBrowserDefault': true});
	});

	AJS.$("#vacationKind").change(function() {
		vacationKindChange(AJS.$(this).val());
	});

	function vacationKindChange(id) {
		vacationKindId=id;
		AJS.$("[name=periodSelectContainer]").hide();
		AJS.$("[name=periodSelectContainer][vacationKindId="+id+"]").show();
		var obj = AJS.$("[name=periodSelect][vacationKindId="+id+"]");
		selectObjectDate(obj, checkDateStart, checkDateEnd);
		var oldCheckStart=checkDateStart;
		var oldCheckEnd=checkDateEnd;
		obj.change();
		checkDateStart=oldCheckStart;
		checkDateEnd=oldCheckEnd;
	}
	
	function selectObjectDate(obj, dateStart, dateEnd) {
		if (obj.prop("tagName").toUpperCase()=="SELECT") {
			var childrens=obj.children();
			var children=null;
			for (var i=0; i<childrens.length; i++) {
				children=AJS.$(childrens[i]);
				var start=new Date(children.val().split(",")[0]);
				var end=new Date(children.val().split(",")[1]);
				if (obj.length>1) {
					start=AJS.$("[name=periodSelectContainer][vacationKindId="+vacationKindId+"]").find(".dateStart[name=periodSelect]").val();
					end=AJS.$("[name=periodSelectContainer][vacationKindId="+vacationKindId+"]").find(".dateEnd[name=periodSelect]").val();
				}
				if (dateStart<=end) {
					obj.val(children.val());
					return;
				}
			}
			obj.val(children.val());
			return;
		} else {
			if (obj.length>1) {
				AJS.$(obj[0]).val(parseDate(dateStart));
				AJS.$(obj[1]).val(parseDate(dateEnd));
			} else {
				AJS.$(obj[0]).val(parseDate(dateStart));
			}
		}
	}

	function parseDate(date) {
		var year=date.getFullYear();
		var month=pad((date.getMonth()+1),2);
		var day=pad((date.getDate()),2);
		return year+"-"+month+"-"+day;
	}
	
	function pad(num, size) {
	    var s = "00000000000000" + num;
	    return s.substr(s.length-size);
	}

	AJS.$('.overlay-spinner').spin();

	AJS.$('[name=periodSelect]').each(function() {
		var obj = AJS.$(this);
		obj.change(function() {
			var dates=obj.val().split(",");
			var start=dates[0];
			var end=dates[0];
			if (dates.length>1) {
				end=dates[1];
			} else {
				start=AJS.$("[name=periodSelectContainer][vacationKindId="+vacationKindId+"]").find(".dateStart[name=periodSelect]").val();
				end=AJS.$("[name=periodSelectContainer][vacationKindId="+vacationKindId+"]").find(".dateEnd[name=periodSelect]").val();
			}
			checkDateStart=new Date(start);
			checkDateEnd=new Date(end);
			changeTableData(AJS.$("#vacationKind").val(), start, end);
		});
	});
	
	function changeTableData(vacationKindId, start, end) {

		statisticsData.vacationKindName = AJS.$("#vacationKind option:selected").text();
		statisticsData.start = start;
		statisticsData.end = end;

		AJS.$("#loading-overlay").show();
		var params={vacationKindId:vacationKindId, startDate:start, endDate:end};
		jQuery.ajax({
			type : "POST",
			url : AJS.contextPath() + "/rest/vacation-manager/1.0/hrmanagement/statistics/instances/vacation-kind",
			data : JSON.stringify(params),
			dataType : "json",
			contentType: 'application/json; charset=utf-8',
			async : true
		}).done(function(data) {
			AJS.$("#loading-overlay").hide();
			populateTableData(data);
		});
	}
	
	function populateTableData(json) {
		var userInstances = json.userInstances;
											
		table.rows().every(function(id) {
			var row=this;
			var data=row.data();
			var userKey=data[TableColumnsEnum.userKey];
			clearRow(row);
			var userInstance=userInstances[userKey];
			if (userInstance!=undefined) {
				var instances=userInstance.instances;
				for (var inst=0; inst<instances.length; inst++) {
					var instance=instances[inst];
					addRowData(row, json.vacationKind, new Date(instance.startDate), new Date(instance.endDate), instance.availableDays, instance.carryOverDays, instance.usedDays, instance.vacationDocuments);
				}
				if (instances.length<=0) {
					addRowData(row, json.vacationKind);
				}
			}
		});
		
		table.draw();
	}
	
	function clearRow(row) {
		var data=row.data();
		
		for (var i=3; i<8; i++) {
			data[i]="";
		}
		
		for (var i=8; i<11; i++) {
			data[i]="0";
		}
		
		data[TableColumnsEnum.documents]="";
		data[TableColumnsEnum.documentsOrderData]="0";
		
		row.data(data);
	}
	
	function addRowData(row, vacationKind, _startDate, _endDate, _availableDays, _carryOverDays, _usedDays, _vacationDocuments) {
		var data=row.data();
		
        var notApplicableLabel="<div value='"+AJS.I18n.getText("vacationmanager.text.not.applicable")+"' class='not-applicable-element'>"+AJS.I18n.getText("vacationmanager.text.not.applicable")+"</div>";
        var noDayLimitLabel="<div value='"+AJS.I18n.getText("vacationmanager.vacationkind.no.day.limit")+"' class='not-applicable-element'>"+AJS.I18n.getText("vacationmanager.vacationkind.no.day.limit")+"</div>";
        var noData="<div value='-' class='not-applicable-element'>-</div>";

		if (_startDate == undefined) {
			_startDate = noData;
		} else {
			_startDate = "<div value='"+VM$.moment(_startDate).format("YYYY-MM-DD")+"'>"+VM$.moment(_startDate).format(JIRA.translateSimpleDateFormat(AJS.Meta.get("date-dmy")))+"</div>";
		}
			
		if (_endDate == undefined) {
			_endDate = noData;
		} else {
			_endDate = "<div value='"+VM$.moment(_endDate).format("YYYY-MM-DD")+"'>"+VM$.moment(_endDate).format(JIRA.translateSimpleDateFormat(AJS.Meta.get("date-dmy")))+"</div>";
			if (vacationKind.validThrough==false && vacationKind.vacationKindType=="ONE_TIME")
				_endDate = notApplicableLabel;
		}

		if (_availableDays == undefined) {
			_availableDays = noData;
			data[TableColumnsEnum.availableDaysOrderData] = NaN;
		} else {
			if (vacationKind.hasDayLimit == false) {
				data[TableColumnsEnum.availableDaysOrderData] = NaN;
				_availableDays=noDayLimitLabel;
			} else {
				data[TableColumnsEnum.availableDaysOrderData]=parseInt(data[TableColumnsEnum.availableDaysOrderData])+parseInt(_availableDays);
				_availableDays="<div value='"+_availableDays+"'>"+_availableDays+"</div>";
			}
		}
		
		if (_carryOverDays == undefined) {
			_carryOverDays = noData;
			data[TableColumnsEnum.carryOverDaysOrderData] = NaN;
		} else {
			if (vacationKind.carryOver == false) {
				data[TableColumnsEnum.carryOverDaysOrderData] = NaN;
				_carryOverDays=notApplicableLabel;
			} else {
				data[TableColumnsEnum.carryOverDaysOrderData]=parseInt(data[TableColumnsEnum.carryOverDaysOrderData])+parseInt(_carryOverDays);
				_carryOverDays="<div value='"+_carryOverDays+"'>"+_carryOverDays+"</div>";
			}
		}
		
		if (_usedDays == undefined) {
			data[TableColumnsEnum.usedDaysOrderData] = NaN;
			_usedDays = noData;
		} else {
			data[TableColumnsEnum.usedDaysOrderData] = parseInt(data[TableColumnsEnum.usedDaysOrderData])+parseInt(_usedDays);
			_usedDays="<div value='"+_usedDays+"'>"+_usedDays+"</div>";
		}

		data[TableColumnsEnum.startDate]+=_startDate;
		data[TableColumnsEnum.endDate]+=_endDate;
		data[TableColumnsEnum.availableDays]+=_availableDays;
		data[TableColumnsEnum.carryOverDays]+=_carryOverDays;
		data[TableColumnsEnum.usedDays]+=_usedDays;

		var documentsCount = 0;
		var documentsLink=VacationManager.Templates.UserStatistics.documentsJqlLink({vacationDocuments:_vacationDocuments,
		    jqlSearchUrl: AJS.params.baseURL + "/issues/?jql="});
		if (documentsLink.trim()=="") {
			documentsLink=noData;
		} else {
			documentsCount = _vacationDocuments.length;
		}
		data[TableColumnsEnum.documents]="<div value='"+documentsCount+"'>"+documentsLink+"</div>";
		data[TableColumnsEnum.documentsOrderData]="";
		if (_vacationDocuments!=undefined) {
			data[TableColumnsEnum.documentsOrderData]=_vacationDocuments.length;
		}
		
		row.data(data);
		row.invalidate();
	}

	var table = VM$('#vacationDefinitionsTable').DataTable({
		language: datatablesLangObject,
		order: [ [TableColumnsEnum.fullname, 'asc'] ],
		columnDefs: [
		   { targets: [TableColumnsEnum.startDate, TableColumnsEnum.endDate, TableColumnsEnum.documents, TableColumnsEnum.documentsOrderData], orderable: false },
		   { targets: [TableColumnsEnum.userKey, TableColumnsEnum.availableDaysOrderData, TableColumnsEnum.carryOverDaysOrderData, TableColumnsEnum.usedDaysOrderData, TableColumnsEnum.documentsOrderData], visible: false},
		   { targets: [TableColumnsEnum.availableDays], orderData: [TableColumnsEnum.availableDaysOrderData] }, //available
		   { targets: [TableColumnsEnum.carryOverDays], orderData: [TableColumnsEnum.carryOverDaysOrderData] }, //carryover
		   { targets: [TableColumnsEnum.usedDays], orderData: [TableColumnsEnum.usedDaysOrderData] } //used
		],
	});
	AJS.$("#vacationDefinitionsTable_filter").hide();

	statisticsData.table = table;
	statisticsData.filter = filter;
	statisticsData.vacationKindName = null;
	statisticsData.start = null;
	statisticsData.end = null;

	vacationKindChange(AJS.$("#vacationKind").val());

	window.onbeforeunload=null;
});
